package jalview.appletgui;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;

public class IdPanel extends Panel implements MouseListener, MouseMotionListener {

  protected IdCanvas       idCanvas;
  protected AlignViewport  av;
  protected AlignmentPanel alignPanel;
  ScrollThread scrollThread = null;

  int            offy;
  int            width;
  int lastid = -1;
  boolean mouseDragging = false;

  public IdPanel(AlignViewport av, AlignmentPanel parent)
  {
    this.av         = av;
    alignPanel = parent;
    idCanvas = new IdCanvas(av);
    setLayout(new BorderLayout());
    add(idCanvas, BorderLayout.CENTER);
    idCanvas.addMouseListener(this);
    idCanvas.addMouseMotionListener(this);
  }

  public void mouseMoved(MouseEvent e) {}


  public void mouseDragged(MouseEvent e) {
    mouseDragging = true;

    int y = e.getY();
    if(av.getWrapAlignment())
      y-=2*av.charHeight;
    int seq = av.getIndex(y);

    if(seq<0)
      return;

    if (seq < lastid)
      selectSeqs(lastid - 1, seq);
    else if (seq > lastid)
      selectSeqs(lastid + 1, seq);

    lastid = seq;
    alignPanel.repaint();
  }

  public void mouseClicked(MouseEvent e)
  {
    if (e.getClickCount() == 2)
    {
       int y = e.getY();
       if(av.getWrapAlignment())
         y-=2*av.charHeight;

       int seq = av.getIndex(y);
       String id = av.getAlignment().getSequenceAt(seq).getName();

       try{
         jalview.bin.JalviewLite.showURL(
             "http://srs.ebi.ac.uk/srs7bin/cgi-bin/wgetz?-e+[swall-id:" + id +
             "]+-vn+2");
       }catch(Exception ex){ex.printStackTrace();}
    }

  }
  public void mouseEntered(MouseEvent e)
  {
    if(scrollThread!=null)
      scrollThread.running = false;
  }

  public void mouseExited (MouseEvent e)
  {
    if(av.getWrapAlignment())
      return;

    if(mouseDragging && e.getY()<0 && av.getStartSeq()>0)
    {
      scrollThread = new ScrollThread(true);
    }

    if(mouseDragging && e.getY()>=getSize().height && av.alignment.getHeight()>av.getEndSeq())
    {
      scrollThread = new ScrollThread(false);
    }
  }


  public void mousePressed(MouseEvent e) {
    if (e.getClickCount() == 2)
      return;

    int y = e.getY();
    if(av.getWrapAlignment())
      y-=2*av.charHeight;

    int seq = av.getIndex(y);
    if (seq == -1)
      return;

    if ( (e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)
    {
      APopupMenu popup = new APopupMenu(alignPanel, null);
      this.add(popup);
      popup.show(this, e.getX(), e.getY());
      return;
    }


    if(!e.isControlDown() && !e.isShiftDown() && av.alignment.findGroup( av.alignment.getSequenceAt(seq))!=null)
    {

      SequenceGroup selection = new SequenceGroup();
      SequenceGroup sg = av.alignment.findGroup(av.alignment.getSequenceAt(seq));
      selection.setStartRes(0);
      selection.setEndRes(av.alignment.getWidth()-1);
      for (int i =0; i< sg.getSize(); i++)
        selection.addSequence(sg.getSequenceAt(i));

      av.setSelectionGroup(selection);
      return;
    }

    if(av.getSelectionGroup()==null || ( !e.isControlDown() && av.getSelectionGroup()!=null))
      av.setSelectionGroup(new SequenceGroup());

    av.getSelectionGroup().setStartRes(0);
    av.getSelectionGroup().setEndRes(av.alignment.getWidth()-1);

    if(e.isShiftDown() && lastid!=-1)
      selectSeqs(lastid, seq);
    else
      selectSeq(seq);

    alignPanel.repaint();
  }

  void selectSeq(int seq)
  {
    lastid = seq;
    SequenceI pickedSeq = av.getAlignment().getSequenceAt(seq);
    av.getSelectionGroup().addOrRemove(pickedSeq);
  }

  void selectSeqs(int start, int end) {

    lastid = start;
    if (end < start)
     {
       int tmp = start;
       start = end;
       end   = tmp;
       lastid = end;
     }

     for (int i = start; i <= end; i++)
        av.getSelectionGroup().addSequence(av.getAlignment().getSequenceAt(i));

   }


  public void mouseReleased(MouseEvent e)
  {
    if(scrollThread!=null)
      scrollThread.running = false;

    mouseDragging = false;
    PaintRefresher.Refresh(this);
  }

  public void highlightSearchResults( java.util.Vector found )
  {
    idCanvas.setHighlighted( found );

    if(found == null)
      return;

    int index = av.alignment.findIndex( (SequenceI)found.elementAt(0));

    // do we need to scroll the panel?
    if(av.getStartSeq()>index || av.getEndSeq()<index)
        alignPanel.setScrollValues( av.getStartRes(), index);
  }

  // this class allows scrolling off the bottom of the visible alignment
  class ScrollThread
      extends Thread
  {
    boolean running = false;
    boolean up = true;
    public ScrollThread(boolean up)
    {
      this.up = up;
      start();
    }

    public void stopScrolling()
    {
      running = false;
    }

    public void run()
    {
      running = true;
      while (running)
      {
        if(alignPanel.scrollUp(up))
        {
          // scroll was ok, so add new sequence to selection
          int seq = av.getStartSeq();
          if(!up)
            seq = av.getEndSeq();

          if (seq < lastid)
            selectSeqs(lastid - 1, seq);
          else if (seq > lastid)
            selectSeqs(lastid + 1, seq);

          lastid = seq;
        }
        else
          running = false;

        alignPanel.repaint();
        try
        {
          Thread.sleep(100);
        }
        catch (Exception ex)
        {}
      }
    }
}

}
